@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -17,5 +17,5 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this software.  If not, see <http://www.gnu.org/licenses/>.
 
-__appname__ = "debspawn"
-__version__ = "0.4.1"
+__appname__ = 'debspawn'
+__version__ = '0.5.0'
