@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -19,11 +19,12 @@
 
 import os
 import subprocess
-import shutil
 import platform
 from glob import glob
-from .utils.env import ensure_root, switch_unprivileged, get_owner_uid_gid, get_free_space, get_tree_size
-from .utils.misc import temp_dir, cd, format_filesize, version_noepoch
+from collections.abc import Iterable
+from .utils.env import ensure_root, switch_unprivileged, get_owner_uid_gid, get_random_free_uid_gid, \
+    get_free_space, get_tree_size
+from .utils.misc import listify, temp_dir, cd, safe_copy, format_filesize, version_noepoch
 from .utils.log import print_header, print_section, print_info, print_warn, print_error, \
     capture_console_output, save_captured_console_output
 from .utils.command import safe_run
@@ -66,8 +67,11 @@ def interact_with_build_environment(osbase, instance_dir, machine_name, *,
     if source_pkg_dir:
         print()
         while True:
-            copy_changes = input(('Should changes to the debian/ directory be copied back to the host?\n'
-                                  'This will OVERRIDE all changes made on files on the host. [y/N]: '))
+            try:
+                copy_changes = input(('Should changes to the debian/ directory be copied back to the host?\n'
+                                      'This will OVERRIDE all changes made on files on the host. [y/N]: '))
+            except EOFError:
+                copy_changes = 'n'
             if copy_changes == 'y' or copy_changes == 'Y':
                 copy_changes = True
                 break
@@ -126,9 +130,7 @@ def interact_with_build_environment(osbase, instance_dir, machine_name, *,
                             os.makedirs(dest_dir, exist_ok=True)
 
                     print('Copy: {}'.format(rel_fname))
-                    shutil.copy2(fname,
-                                 dest_fname,
-                                 follow_symlinks=False)
+                    safe_copy(fname, dest_fname)
                     os.chown(dest_fname, o_uid, o_gid, follow_symlinks=False)
 
             for rel_fname, fname in known_files.items():
@@ -142,12 +144,18 @@ def interact_with_build_environment(osbase, instance_dir, machine_name, *,
 
 
 def internal_execute_build(osbase, pkg_dir, build_only=None, *,
-                           qa_lintian=False, interact=False, source_pkg_dir=None, buildflags=[]):
+                           qa_lintian=False, interact=False, source_pkg_dir=None,
+                           buildflags: list[str] = None, build_env: dict[str, str] = None):
     ''' Perform the actual build on an extracted package directory '''
     assert not build_only or isinstance(build_only, str)
     if not pkg_dir:
         raise Exception('Package directory is missing!')
     pkg_dir = os.path.normpath(pkg_dir)
+    if not build_env:
+        build_env = {}
+
+    # get a fresh UID to give to our build user within the container
+    builder_uid = get_random_free_uid_gid()[0]
 
     with osbase.new_instance() as (instance_dir, machine_name):
         # first, check basic requirements
@@ -183,6 +191,7 @@ def internal_execute_build(osbase, pkg_dir, build_only=None, *,
                                           machine_name,
                                           prep_flags,
                                           '/srv',
+                                          build_uid=builder_uid,
                                           nspawn_flags=nspawn_flags,
                                           tmp_apt_cache_dir=aptcache_tmp,
                                           pkginjector=pkginjector)
@@ -203,9 +212,11 @@ def internal_execute_build(osbase, pkg_dir, build_only=None, *,
                                           machine_name,
                                           helper_flags,
                                           '/srv',
+                                          build_uid=builder_uid,
                                           nspawn_flags=nspawn_flags,
                                           tmp_apt_cache_dir=aptcache_tmp,
                                           pkginjector=pkginjector,
+                                          env_vars=build_env,
                                           syscall_filter=osbase.global_config.syscall_filter)
             # exit, unless we are in interactive mode
             if r != 0 and not interact:
@@ -221,6 +232,7 @@ def internal_execute_build(osbase, pkg_dir, build_only=None, *,
                                               machine_name,
                                               ['--run-qa', '--lintian'],
                                               '/srv',
+                                              build_uid=builder_uid,
                                               nspawn_flags=nspawn_flags,
                                               tmp_apt_cache_dir=aptcache_tmp,
                                               pkginjector=pkginjector)
@@ -238,7 +250,7 @@ def internal_execute_build(osbase, pkg_dir, build_only=None, *,
                                                 aptcache_tmp=aptcache_tmp,
                                                 pkginjector=pkginjector,
                                                 prev_exitcode=r)
-                # exit with status of previous exist code
+                # exit with status of previous exit code
                 if r != 0:
                     return False
 
@@ -279,9 +291,11 @@ def _read_source_package_details():
     return pkg_sourcename, pkg_version, dsc_fname
 
 
-def _get_build_flags(build_only=None, include_orig=False, maintainer=None, extra_flags=[]):
+def _get_build_flags(build_only=None, include_orig=False, maintainer=None,
+                     extra_flags: Iterable[str] = None):
     import shlex
     buildflags = []
+    extra_flags = listify(extra_flags)
 
     if build_only:
         if build_only == 'binary':
@@ -315,7 +329,7 @@ def _retrieve_artifacts(osbase, tmp_dir):
     for f in glob(os.path.join(tmp_dir, '*.*')):
         if os.path.isfile(f):
             target_fname = os.path.join(osbase.results_dir, os.path.basename(f))
-            shutil.copy2(f, target_fname)
+            safe_copy(f, target_fname)
             os.chown(target_fname, o_uid, o_gid, follow_symlinks=False)
             acount += 1
     print_info('Copied {} files.'.format(acount))
@@ -327,7 +341,7 @@ def _sign_result(results_dir, spkg_name, spkg_version, build_arch):
     changes_basename = '{}_{}_{}.changes'.format(spkg_name, spkg_version_noepoch, build_arch)
 
     with switch_unprivileged():
-        proc = subprocess.run(['debsign', os.path.join(results_dir, changes_basename)])
+        proc = subprocess.run(['debsign', os.path.join(results_dir, changes_basename)], check=False)
         if proc.returncode != 0:
             print_error('Signing failed.')
             return False
@@ -337,14 +351,18 @@ def _sign_result(results_dir, spkg_name, spkg_version, build_arch):
 def _print_system_info():
     from . import __version__
     from .utils.misc import current_time_string
-    print_info('debspawn {version} on {host} at {time}'.format(version=__version__, host=platform.node(), time=current_time_string()))
+    print_info('debspawn {version} on {host} at {time}'.format(version=__version__,
+                                                               host=platform.node(),
+                                                               time=current_time_string()))
 
 
 def build_from_directory(osbase, pkg_dir, *,
                          sign=False, build_only=None, include_orig=False, maintainer=None,
-                         clean_source=False, qa_lintian=False, interact=False, log_build=True, extra_dpkg_flags=[]):
+                         clean_source=False, qa_lintian=False, interact=False, log_build=True,
+                         extra_dpkg_flags: list[str] = None, build_env: dict[str, str] = None):
     ensure_root()
     osbase.ensure_exists()
+    extra_dpkg_flags = listify(extra_dpkg_flags)
 
     if interact and log_build:
         print_warn('Build log and interactive mode can not be enabled at the same time. Disabling build log.')
@@ -383,7 +401,7 @@ def build_from_directory(osbase, pkg_dir, *,
             if not clean_source:
                 cmd.append('-nc')
 
-            proc = subprocess.run(cmd)
+            proc = subprocess.run(cmd, check=False)
             if proc.returncode != 0:
                 return False
 
@@ -398,31 +416,38 @@ def build_from_directory(osbase, pkg_dir, *,
     print_header('Package build')
     print_build_detail(osbase, pkg_sourcename, pkg_version)
 
+    success = False
     with temp_dir(pkg_sourcename) as pkg_tmp_dir:
         with cd(pkg_tmp_dir):
             cmd = ['dpkg-source',
                    '-x', os.path.join(pkg_dir, '..', dsc_fname)]
-            proc = subprocess.run(cmd)
+            proc = subprocess.run(cmd, check=False)
             if proc.returncode != 0:
                 return False
 
-        ret = internal_execute_build(osbase,
-                                     pkg_tmp_dir,
-                                     build_only,
-                                     qa_lintian=qa_lintian,
-                                     interact=interact,
-                                     source_pkg_dir=pkg_dir,
-                                     buildflags=buildflags)
-        if not ret:
-            return False
+        success = internal_execute_build(osbase,
+                                         pkg_tmp_dir,
+                                         build_only,
+                                         qa_lintian=qa_lintian,
+                                         interact=interact,
+                                         source_pkg_dir=pkg_dir,
+                                         buildflags=buildflags,
+                                         build_env=build_env)
 
         # copy build results
-        _retrieve_artifacts(osbase, pkg_tmp_dir)
+        if success:
+            _retrieve_artifacts(osbase, pkg_tmp_dir)
 
     # save buildlog, if we generated one
-    log_fname = os.path.join(osbase.results_dir, '{}_{}_{}.buildlog'.format(pkg_sourcename, version_noepoch(pkg_version), osbase.arch))
+    log_fname = os.path.join(osbase.results_dir, '{}_{}_{}.buildlog'.format(pkg_sourcename,
+                                                                            version_noepoch(pkg_version),
+                                                                            osbase.arch))
     save_captured_console_output(log_fname)
 
+    # exit, there is nothing more to do if no package was built
+    if not success:
+        return False
+
     # sign the resulting package
     if sign:
         r = _sign_result(osbase.results_dir, pkg_sourcename, pkg_version, osbase.arch)
@@ -436,9 +461,11 @@ def build_from_directory(osbase, pkg_dir, *,
 
 def build_from_dsc(osbase, dsc_fname, *,
                    sign=False, build_only=None, include_orig=False, maintainer=None,
-                   qa_lintian=False, interact=False, log_build=True, extra_dpkg_flags=[]):
+                   qa_lintian: bool = False, interact: bool = False, log_build: bool = True,
+                   extra_dpkg_flags: Iterable[str] = None, build_env: dict[str, str] = None):
     ensure_root()
     osbase.ensure_exists()
+    extra_dpkg_flags = listify(extra_dpkg_flags)
 
     if interact and log_build:
         print_warn('Build log and interactive mode can not be enabled at the same time. Disabling build log.')
@@ -455,13 +482,14 @@ def build_from_dsc(osbase, dsc_fname, *,
 
     _print_system_info()
 
+    success = False
     dsc_fname = os.path.abspath(os.path.normpath(dsc_fname))
     tmp_prefix = os.path.basename(dsc_fname).replace('.dsc', '').replace(' ', '-')
     with temp_dir(tmp_prefix) as pkg_tmp_dir:
         with cd(pkg_tmp_dir):
             cmd = ['dpkg-source',
                    '-x', dsc_fname]
-            proc = subprocess.run(cmd)
+            proc = subprocess.run(cmd, check=False)
             if proc.returncode != 0:
                 return False
 
@@ -482,22 +510,28 @@ def build_from_dsc(osbase, dsc_fname, *,
             print_header('Package build')
             print_build_detail(osbase, pkg_sourcename, pkg_version)
 
-        ret = internal_execute_build(osbase,
-                                     pkg_tmp_dir,
-                                     build_only,
-                                     qa_lintian=qa_lintian,
-                                     interact=interact,
-                                     buildflags=buildflags)
-        if not ret:
-            return False
+        success = internal_execute_build(osbase,
+                                         pkg_tmp_dir,
+                                         build_only,
+                                         qa_lintian=qa_lintian,
+                                         interact=interact,
+                                         buildflags=buildflags,
+                                         build_env=build_env)
 
         # copy build results
-        _retrieve_artifacts(osbase, pkg_tmp_dir)
+        if success:
+            _retrieve_artifacts(osbase, pkg_tmp_dir)
 
     # save buildlog, if we generated one
-    log_fname = os.path.join(osbase.results_dir, '{}_{}_{}.buildlog'.format(pkg_sourcename, version_noepoch(pkg_version), osbase.arch))
+    log_fname = os.path.join(osbase.results_dir, '{}_{}_{}.buildlog'.format(pkg_sourcename,
+                                                                            version_noepoch(pkg_version),
+                                                                            osbase.arch))
     save_captured_console_output(log_fname)
 
+    # build log is saved, but no artifacts are available, so there's nothing more to do
+    if not success:
+        return False
+
     # sign the resulting package
     if sign:
         r = _sign_result(osbase.results_dir, pkg_sourcename, pkg_version, osbase.arch)
@@ -505,5 +539,4 @@ def build_from_dsc(osbase, dsc_fname, *,
             return False
 
     print_info('Done.')
-
     return True
