@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -38,8 +38,11 @@ def init_config(options):
     if options.no_unicode:
         set_unicode_allowed(False)
     else:
-        if 'utf-8' not in os.environ.get('LANG', 'utf-8').lower():
-            log.warning('Building with unicode support, but your environment does not seem to support unicode.')
+        import locale
+        current_encoding = locale.getpreferredencoding()
+        if current_encoding.lower() != 'utf-8':
+            log.warning(('Building with unicode support, but your environment does not seem to support unicode. '
+                         '(Encoding is {})').format(current_encoding))
         set_unicode_allowed(True)
 
     if options.owner:
@@ -89,8 +92,9 @@ def command_create(options):
                     base_suite=options.base_suite)
     r = osbase.create(options.mirror,
                       components,
-                      extra_suites,
-                      options.extra_source_lines)
+                      extra_suites=extra_suites,
+                      extra_source_lines=options.extra_source_lines,
+                      allow_recommends=options.allow_recommends)
     if not r:
         sys.exit(2)
 
@@ -148,6 +152,18 @@ def command_build(options):
     gconf = init_config(options)
     osbase = OSBase(gconf, options.suite, options.arch, options.variant)
 
+    # prepare user-defined environment variables
+    env_vars = {}
+    if options.env_vars:
+        for kv in options.env_vars:
+            p = kv.split('=', 1)
+            if len(p) != 2:
+                print('Environment variable definition `{}` is invalid!'.format(kv))
+                print('Can not continue.')
+                sys.exit(1)
+            env_vars[p[0]] = p[1]
+
+    # prepare user-defined buildflags
     buildflags = []
     if options.buildflags:
         buildflags = options.buildflags.split(';')
@@ -173,7 +189,8 @@ def command_build(options):
                                  qa_lintian=options.lintian,
                                  interact=options.interact,
                                  log_build=not options.no_buildlog,
-                                 extra_dpkg_flags=buildflags)
+                                 extra_dpkg_flags=buildflags,
+                                 build_env=env_vars)
     else:
         r = build_from_dsc(osbase,
                            options.target,
@@ -184,7 +201,8 @@ def command_build(options):
                            qa_lintian=options.lintian,
                            interact=options.interact,
                            log_build=not options.no_buildlog,
-                           extra_dpkg_flags=buildflags)
+                           extra_dpkg_flags=buildflags,
+                           build_env=env_vars)
     if not r:
         sys.exit(2)
 
@@ -236,6 +254,37 @@ def command_run(options, custom_command):
         sys.exit(2)
 
 
+def command_maintain(options):
+    ''' Execute global maintenance actions '''
+
+    check_print_version(options)
+    gconf = init_config(options)
+
+    if options.migrate:
+        from .maintain import maintain_migrate
+        maintain_migrate(gconf)
+        return
+    if options.clear_caches:
+        from .maintain import maintain_clear_caches
+        maintain_clear_caches(gconf)
+        return
+    if options.update_all:
+        from .maintain import maintain_update_all
+        maintain_update_all(gconf)
+        return
+    if options.purge:
+        from .maintain import maintain_purge
+        maintain_purge(gconf, options.yes)
+        return
+    if options.status:
+        from .maintain import maintain_print_status
+        maintain_print_status(gconf)
+        return
+
+    print('No maintenance action selected!')
+    sys.exit(1)
+
+
 class CustomArgparseFormatter(HelpFormatter):
 
     def _split_lines(self, text, width):
@@ -284,11 +333,17 @@ def create_parser(formatter_class=None):
     sp.add_argument('--components', action='store', dest='components', default=None,
                     help='A comma-separated list of archive components to enable in the newly created image.')
     sp.add_argument('--base-suite', action='store', dest='base_suite', default=None,
-                    help='A full suite that forms the base of the selected partial suite (e.g. for -updates and -backports).')
+                    help=('A full suite that forms the base of the selected partial suite '
+                          '(e.g. for -updates and -backports).'))
     sp.add_argument('--extra-suites', action='store', dest='extra_suites', default=None,
-                    help='Space-separated list of additional suites that should also be added to the sources.list file.')
+                    help=('Space-separated list of additional suites that should also be added to the '
+                          'sources.list file.'))
     sp.add_argument('--extra-sourceslist-lines', action='store', dest='extra_source_lines', default=None,
-                    help='Lines that should be added to the build environments source.list verbatim. Separate lines by linebreaks.')
+                    help=('Lines that should be added to the build environments source.list verbatim. '
+                          'Separate lines by linebreaks.'))
+    sp.add_argument('--allow-recommends', action='store', dest='allow_recommends', default=None,
+                    help=('Do not disable APT installing "recommends"-type dependencies by default, and instead use '
+                          'the default behavior for full, normal system installations with "recommends" enabled.'))
     sp.set_defaults(func=command_create)
 
     # 'delete' command
@@ -300,7 +355,8 @@ def create_parser(formatter_class=None):
     sp = subparsers.add_parser('update', help='Update a container image')
     add_container_select_arguments(sp)
     sp.add_argument('--recreate', action='store_true', dest='recreate',
-                    help='Re-create the container image from scratch using the settings used to create it previously, instead of just updating it.')
+                    help=('Re-create the container image from scratch using the settings used to create it previously, '
+                          'instead of just updating it.'))
     sp.set_defaults(func=command_update)
 
     # 'list' command
@@ -308,7 +364,8 @@ def create_parser(formatter_class=None):
     sp.set_defaults(func=command_list)
 
     # 'build' command
-    sp = subparsers.add_parser('build', help='Build a package in an isolated environment', formatter_class=formatter_class, aliases=['b'])
+    sp = subparsers.add_parser('build', help='Build a package in an isolated environment',
+                               formatter_class=formatter_class, aliases=['b'])
     add_container_select_arguments(sp)
     sp.add_argument('-s', '--sign', action='store_true', dest='sign',
                     help='Sign the resulting package.')
@@ -325,17 +382,22 @@ def create_parser(formatter_class=None):
     sp.add_argument('--results-dir', action='store', dest='results_dir',
                     help='Override the configured results directory and return artifacts at a custom location.')
     sp.add_argument('--maintainer', action='store', dest='maintainer',
-                    help=('Set the name and email address of the maintainer for this package and upload, rather than using '
-                          'the information from the source tree\'s control file or changelog.'))
+                    help=('Set the name and email address of the maintainer for this package and upload, rather than '
+                          'using the information from the source tree\'s control file or changelog.'))
     sp.add_argument('--clean-source', action='store_true', dest='clean_source',
-                    help=('Run the d/rules clean target outside of the container. This means the package build dependencies need to be '
-                          'installed on the host system when building from a source directory.'))
+                    help=('Run the d/rules clean target outside of the container. This means the package build '
+                          'dependencies need to be installed on the host system when building from a '
+                          'source directory.'))
     sp.add_argument('--lintian', action='store_true', dest='lintian',
                     help='Run the Lintian static analysis tool for Debian packages after the package is built.')
     sp.add_argument('--no-buildlog', action='store_true', dest='no_buildlog',
                     help='Do not write a build log.')
     sp.add_argument('-i', '--interact', action='store_true', dest='interact',
-                    help='Run an interactive shell in the build environment after build. This implies `--no-buildlog` and disables the log.')
+                    help=('Run an interactive shell in the build environment after build. This implies `--no-buildlog` '
+                          'and disables the log.'))
+    sp.add_argument('-e', '--setenv', action='append', dest='env_vars',
+                    help=('Set an environment variable for the build process. Takes a `key=value` pair any may be '
+                          'defined multiple times to set different environment variables.'))
     sp.add_argument('target', action='store', nargs='?', default=None,
                     help='The source package file or source directory to build.')
     sp.set_defaults(func=command_build)
@@ -346,7 +408,8 @@ def create_parser(formatter_class=None):
     sp.add_argument('--persistent', action='store_true', dest='persistent',
                     help='Make changes done in the session persistent.')
     sp.add_argument('--allow', action='store', dest='allow',
-                    help='List one or more additional permissions to grant the container. Takes a comma-separated list of capability names.')
+                    help=('List one or more additional permissions to grant the container. Takes a comma-separated '
+                          'list of capability names.'))
     sp.set_defaults(func=command_login)
 
     # 'run' command
@@ -357,19 +420,39 @@ def create_parser(formatter_class=None):
     sp.add_argument('--build-dir', action='store', dest='build_dir', default=None,
                     help='Select a host directory that gets bind mounted to /srv/build.')
     sp.add_argument('--cachekey', action='store', dest='cachekey', default=None,
-                    help=('If set, use the specified cache-ID to store an initialized container image for faster initialization times.\n'
-                          'This may mean that the command passed in `--init-command` is skipped if the cache already existed.'))
+                    help=('If set, use the specified cache-ID to store an initialized container image for faster '
+                          'initialization times.\n'
+                          'This may mean that the command passed in `--init-command` is skipped if the cache '
+                          'already existed.'))
     sp.add_argument('--init-command', action='store', dest='init_command', default=None,
                     help='The command or command script used to set up the container.')
     sp.add_argument('-x', '--external-command', action='store_true', dest='external_commad',
-                    help='If set, the command script(s) will be copied from the host to the container and then executed.')
+                    help=('If set, the command script(s) will be copied from the host to the container '
+                          'and then executed.'))
     sp.add_argument('--header', action='store', dest='header', default=None,
                     help='Name of the task that is run, will be printed as header.')
     sp.add_argument('--allow', action='store', dest='allow',
-                    help='List one or more additional permissions to grant the container. Takes a comma-separated list of capability names.')
+                    help=('List one or more additional permissions to grant the container. Takes a comma-separated '
+                          'list of capability names.'))
     sp.add_argument('command', action='store', nargs='*', default=None,
                     help='The command to run.')
 
+    # 'maintain' command
+    sp = subparsers.add_parser('maintain', help='Execute various maintenance actions, affecting all images')
+    sp.add_argument('-y', '--yes', action='store_true', dest='yes',
+                    help='Perform dangerous actions without asking twice.')
+    sp.add_argument('--migrate', action='store_true', dest='migrate',
+                    help='Migrate any settings or configuration changes to the current version of debspawn.')
+    sp.add_argument('--update-all', action='store_true', dest='update_all',
+                    help='Update all container images that we know.')
+    sp.add_argument('--clear-caches', action='store_true', dest='clear_caches',
+                    help='Delete all cached packages for all images.')
+    sp.add_argument('--purge', action='store_true', dest='purge',
+                    help='Remove all images as well as any data associated with them.')
+    sp.add_argument('--status', action='store_true', dest='status',
+                    help='Display a status summary about this installation, highlighting potential issues.')
+    sp.set_defaults(func=command_maintain)
+
     return parser
 
 
