@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -53,10 +53,11 @@ class GlobalConfig:
 
             self._dsrun_path = os.path.normpath(os.path.join(thisfile, '..', 'dsrun'))
             if not os.path.isfile(self._dsrun_path):
-                print('Debspawn is not set up properly: Unable to find file "{}". Can not continue.'.format(self._dsrun_path), file=sys.stderr)
+                print('Debspawn is not set up properly: Unable to find file "{}". Can not continue.'.format(
+                    self._dsrun_path), file=sys.stderr)
                 sys.exit(4)
 
-            self._osroots_dir = cdata.get('OSImagesDir', '/var/lib/debspawn/containers/')
+            self._osroots_dir = cdata.get('OSImagesDir', '/var/lib/debspawn/images/')
             self._results_dir = cdata.get('ResultsDir', '/var/lib/debspawn/results/')
             self._aptcache_dir = cdata.get('APTCacheDir', '/var/lib/debspawn/aptcache/')
             self._injected_pkgs_dir = cdata.get('InjectedPkgsDir', '/var/lib/debspawn/injected-pkgs/')
@@ -76,8 +77,9 @@ class GlobalConfig:
                 self._syscall_filter = []
             else:
                 if type(self._syscall_filter) is not list:
-                    print('Configuration error (global.toml): Entry "SyscallFilter" needs to be either a string value ("compat" or "nspawn-default"), ' +
-                          'or a list of permissible system call names as listed by the syscall-filter command of systemd-analyze(1)', file=sys.stderr)
+                    print(('Configuration error (global.toml): Entry "SyscallFilter" needs to be either a string value '
+                           '("compat" or "nspawn-default"), or a list of permissible system call names as listed by '
+                           'the syscall-filter command of systemd-analyze(1)'), file=sys.stderr)
                     sys.exit(8)
 
         @property
@@ -85,7 +87,7 @@ class GlobalConfig:
             return self._dsrun_path
 
         @dsrun_path.setter
-        def dsrun_path(self, v) -> str:
+        def dsrun_path(self, v):
             self._dsrun_path = v
 
         @property
