@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -19,8 +19,11 @@
 
 import os
 import platform
+from typing import Union
 from .utils import temp_dir, print_error, print_warn, print_info, safe_run, run_forwarded
 from .utils.env import colored_output_allowed, unicode_allowed
+from .utils.command import run_command
+from .injectpkg import PackageInjector
 
 
 __systemd_version = None
@@ -43,6 +46,17 @@ def systemd_version():
     return __systemd_version
 
 
+def systemd_detect_virt():
+    vm_name = 'unknown'
+    try:
+        out, _, _ = run_command(['systemd-detect-virt'])
+        vm_name = out.strip()
+    except Exception as e:
+        print_warn('Unable to determine current virtualization: {}'.format(e))
+
+    return vm_name
+
+
 def systemd_version_atleast(expected_version: int):
     v = systemd_version()
     # we always assume we are running the highest version,
@@ -67,18 +81,27 @@ def get_nspawn_personality(osbase):
     return None
 
 
-def _execute_sdnspawn(osbase, parameters, machine_name, allow_permissions=[], syscall_filter=[]):
+def _execute_sdnspawn(osbase, parameters, machine_name, *,
+                      allow_permissions: list[str] = None, syscall_filter: list[str] = None,
+                      env_vars: dict[str, str] = None, private_users: bool = False):
     '''
     Execute systemd-nspawn with the given parameters.
     Mess around with cgroups if necessary.
     '''
     import sys
-    import time
+
+    if not allow_permissions:
+        allow_permissions = []
+    if not syscall_filter:
+        syscall_filter = []
+    if not env_vars:
+        env_vars = {}
 
     capabilities = []
     full_dev_access = False
     full_proc_access = False
     ro_kmods_access = False
+    kvm_access = False
     all_privileges = False
     for perm in allow_permissions:
         perm = perm.lower()
@@ -95,19 +118,33 @@ def _execute_sdnspawn(osbase, parameters, machine_name, allow_permissions=[], sy
             full_proc_access = True
         elif perm == 'read-kmods':
             ro_kmods_access = True
+        elif perm == 'kvm':
+            kvm_access = True
         else:
             print_info('Unknown allowed permission: {}'.format(perm))
 
-    if (capabilities or full_dev_access or full_proc_access) and not osbase.global_config.allow_unsafe_perms:
-        print_error('Configuration does not permit usage of additional and potentially dangerous additional permissions. Exiting.')
+    if (capabilities or full_dev_access or full_proc_access or kvm_access) \
+            and not osbase.global_config.allow_unsafe_perms:
+        print_error('Configuration does not permit usage of additional and potentially dangerous permissions. Exiting.')
         sys.exit(9)
 
     cmd = ['systemd-nspawn']
     cmd.extend(['-M', machine_name])
+    cmd.append('--register=no')
+    if private_users:
+        cmd.append('-U')  # User namespaces with --private-users=pick --private-users-chown, if possible
     if full_dev_access:
         cmd.extend(['--bind', '/dev'])
         if systemd_version_atleast(244):
             cmd.append('--console=pipe')
+        cmd.extend(['--property=DeviceAllow=block-* rw',
+                    '--property=DeviceAllow=char-* rw'])
+    if kvm_access and not full_dev_access:
+        if os.path.exists('/dev/kvm'):
+            cmd.extend(['--bind', '/dev/kvm'])
+            cmd.extend(['--property=DeviceAllow=/dev/kvm rw'])
+        else:
+            print_warn('Access to KVM requested, but /dev/kvm does not exist on the host. Is virtualization supported?')
     if full_proc_access:
         cmd.extend(['--bind', '/proc'])
         if not all_privileges:
@@ -118,57 +155,30 @@ def _execute_sdnspawn(osbase, parameters, machine_name, allow_permissions=[], sy
         cmd.extend(['--capability', ','.join(capabilities)])
     if syscall_filter:
         cmd.extend(['--system-call-filter', ' '.join(syscall_filter)])
+
+    for v_name, v_value in env_vars.items():
+        cmd.extend(['-E', '{}={}'.format(v_name, v_value)])
+
     cmd.extend(parameters)
 
-    if not full_dev_access:
-        proc = run_forwarded(cmd)
-        return proc.returncode
-    else:
-        out, _, _ = safe_run(['systemd-escape', machine_name])
-        escaped_full_machine_name = out.strip()
-
-        pid = os.fork()
-        if pid == 0:
-            # child process - edit the cgroup to allow full access to all
-            # devices. Hopefully there won't be too much need for this awful code.
-            parent_pid = os.getppid()
-            if not all_privileges:
-                print_warn('Container will have direct access to all host devices.')
-
-            syscg_devices_allow = '/sys/fs/cgroup/devices/machine.slice/machine-{}.scope/devices.allow'.format(escaped_full_machine_name)
-            tries = 0
-            while not os.path.exists(syscg_devices_allow):
-                time.sleep(0.5)
-                tries += 1
-                if tries > 40:
-                    break
-
-                # check if our parent process has died - this is very prone to race conditions, but
-                # the best simple way to perform this check. The Linux kernel has neat features to
-                # make this easier though.
-                if os.getppid() != parent_pid:
-                    os._exit(0)
-
-            if not os.path.isfile(syscg_devices_allow):
-                print_error('Unable to give container full read/write permissions on host /dev!')
-                os._exit(0)
-            with open(syscg_devices_allow, 'w') as sys_f:
-                sys_f.write('c *:* rwm\n')  # full access to character devices
-                sys_f.flush()
-                sys_f.write('b *:* rwm\n')  # full access to block devices
-                sys_f.flush()
-            os._exit(0)
-        else:
-            proc = run_forwarded(cmd)
-            return proc.returncode
+    proc = run_forwarded(cmd)
+    return proc.returncode
 
 
-def nspawn_run_persist(osbase, base_dir, machine_name, chdir, command=[], flags=[], *,
-                       tmp_apt_cache_dir=None, pkginjector=None, allowed=[], syscall_filter=[], verbose=False):
+def nspawn_run_persist(osbase, base_dir, machine_name, chdir,
+                       command: Union[list[str], str] = None, flags: Union[list[str], str] = None, *,
+                       tmp_apt_cache_dir: str = None, pkginjector: PackageInjector = None,
+                       allowed: list[str] = None, syscall_filter: list[str] = None,
+                       env_vars: dict[str, str] = None, private_users: bool = False,
+                       verbose: bool = False):
     if isinstance(command, str):
         command = command.split(' ')
+    elif not command:
+        command = []
     if isinstance(flags, str):
         flags = flags.split(' ')
+    elif not flags:
+        flags = []
 
     personality = get_nspawn_personality(osbase)
 
@@ -189,7 +199,13 @@ def nspawn_run_persist(osbase, base_dir, machine_name, chdir, command=[], flags=
         osbase.aptcache.create_instance_cache(aptcache_tmp_dir)
 
         # run command in container
-        ret = _execute_sdnspawn(osbase, params, machine_name, allowed, syscall_filter)
+        ret = _execute_sdnspawn(osbase,
+                                params,
+                                machine_name,
+                                allow_permissions=allowed,
+                                syscall_filter=syscall_filter,
+                                env_vars=env_vars,
+                                private_users=private_users)
 
         # archive APT cache, so future runs of this command are faster
         osbase.aptcache.merge_from_dir(aptcache_tmp_dir)
@@ -205,11 +221,18 @@ def nspawn_run_persist(osbase, base_dir, machine_name, chdir, command=[], flags=
     return ret
 
 
-def nspawn_run_ephemeral(osbase, base_dir, machine_name, chdir, command=[], flags=[], allowed=[], syscall_filter=[]):
+def nspawn_run_ephemeral(osbase, base_dir, machine_name, chdir,
+                         command: Union[list[str], str] = None, flags: Union[list[str], str] = None,
+                         allowed: list[str] = None, syscall_filter: list[str] = None,
+                         env_vars: dict[str, str] = None, private_users: bool = False):
     if isinstance(command, str):
         command = command.split(' ')
+    elif not command:
+        command = []
     if isinstance(flags, str):
         flags = flags.split(' ')
+    elif not flags:
+        flags = []
 
     personality = get_nspawn_personality(osbase)
 
@@ -221,10 +244,16 @@ def nspawn_run_ephemeral(osbase, base_dir, machine_name, chdir, command=[], flag
     params.extend(['-aqxD', base_dir])
     params.extend(command)
 
-    return _execute_sdnspawn(osbase, params, machine_name, allowed, syscall_filter)
+    return _execute_sdnspawn(osbase,
+                             params,
+                             machine_name,
+                             allow_permissions=allowed,
+                             syscall_filter=syscall_filter,
+                             env_vars=env_vars,
+                             private_users=private_users)
 
 
-def nspawn_make_helper_cmd(flags):
+def nspawn_make_helper_cmd(flags, build_uid: int):
     if isinstance(flags, str):
         flags = flags.split(' ')
 
@@ -233,24 +262,33 @@ def nspawn_make_helper_cmd(flags):
         cmd.append('--no-color')
     if not unicode_allowed():
         cmd.append('--no-unicode')
+    if build_uid > 0:
+        cmd.append('--buid={}'.format(build_uid))
 
     cmd.extend(flags)
     return cmd
 
 
-def nspawn_run_helper_ephemeral(osbase, base_dir, machine_name, helper_flags, chdir='/tmp', *, nspawn_flags=[], allowed=[]):
-    cmd = nspawn_make_helper_cmd(helper_flags)
+def nspawn_run_helper_ephemeral(osbase, base_dir, machine_name, helper_flags, chdir='/tmp', *,
+                                build_uid: int, nspawn_flags: Union[list[str], str] = None,
+                                allowed: list[str] = None, env_vars: dict[str, str] = None,
+                                private_users: bool = False):
+    cmd = nspawn_make_helper_cmd(helper_flags, build_uid)
     return nspawn_run_ephemeral(base_dir,
                                 machine_name,
                                 chdir,
                                 cmd,
-                                nspawn_flags,
-                                allowed)
+                                flags=nspawn_flags,
+                                allowed=allowed,
+                                env_vars=env_vars,
+                                private_users=private_users)
 
 
 def nspawn_run_helper_persist(osbase, base_dir, machine_name, helper_flags, chdir='/tmp', *,
-                              nspawn_flags=[], tmp_apt_cache_dir=None, pkginjector=None, allowed=[], syscall_filter=[]):
-    cmd = nspawn_make_helper_cmd(helper_flags)
+                              build_uid: int, nspawn_flags=[], tmp_apt_cache_dir=None,
+                              pkginjector=None, allowed: list[str] = None, syscall_filter: list[str] = None,
+                              env_vars: dict[str, str] = None, private_users: bool = False):
+    cmd = nspawn_make_helper_cmd(helper_flags, build_uid)
     return nspawn_run_persist(osbase,
                               base_dir,
                               machine_name,
@@ -260,4 +298,6 @@ def nspawn_run_helper_persist(osbase, base_dir, machine_name, helper_flags, chdi
                               tmp_apt_cache_dir=tmp_apt_cache_dir,
                               pkginjector=pkginjector,
                               allowed=allowed,
-                              syscall_filter=syscall_filter)
+                              syscall_filter=syscall_filter,
+                              env_vars=env_vars,
+                              private_users=private_users)
