@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -23,15 +23,29 @@ import subprocess
 import shutil
 from pathlib import Path
 from contextlib import contextmanager
+from typing import Optional
 from .utils import temp_dir, print_header, print_section, format_filesize, \
     print_info, print_error, print_warn, listify
-from .utils.env import ensure_root
+from .utils.misc import maybe_remove
+from .utils.env import ensure_root, get_random_free_uid_gid
 from .utils.command import safe_run
 from .utils.zstd_tar import compress_directory, decompress_tarball, ensure_tar_zstd
 from .nspawn import nspawn_run_helper_persist, nspawn_run_persist
 from .aptcache import APTCache
 
 
+def debootstrap_version():
+    ds_version = 'unknown'
+    try:
+        out, _, _ = safe_run(['debootstrap', '--version'])
+        parts = out.strip().split(' ', 2)
+        ds_version = parts[0 if len(parts) < 2 else 1]
+    except Exception as e:
+        print_warn('Unable to determine debootstrap version: {}'.format(e))
+
+    return ds_version
+
+
 class OSBase:
     '''
     Describes an OS base registered with debspawn
@@ -51,14 +65,17 @@ class OSBase:
 
         self._aptcache = APTCache(self)
 
+        # get a fresh UID to give to our build user within the container
+        self._builder_uid = get_random_free_uid_gid()[0]
+
         # ensure we can (de)compress zstd tarballs
         ensure_tar_zstd()
 
     def _make_name(self):
         if not self._arch:
-            out, _, ret = safe_run(['dpkg-architecture', '-qDEB_HOST_ARCH'])
+            out, _, ret = safe_run(['dpkg', '--print-architecture'])
             if ret != 0:
-                raise Exception('Running dpkg-architecture failed: {}'.format(out))
+                raise Exception('Running dpkg --print-architecture failed: {}'.format(out))
 
             self._arch = out.strip()
         if self._variant:
@@ -208,12 +225,30 @@ class OSBase:
                 if os.path.lexists(fname) and not os.path.isdir(fname) and not os.path.ismount(fname):
                     os.remove(fname)
 
-    def _create_internal(self, mirror=None, components=None, extra_suites=[], extra_source_lines=None, show_header=True):
+    def _remove_unwanted_files(self, instance_dir):
+        ''' Delete unwanted files from a base image '''
+
+        # drop resolv.conf: Some OSes set this to a symlink, which will lead to nowhere
+        # in the container and may cause issues when bindmounting over it
+        maybe_remove(os.path.join(instance_dir, 'etc', 'resolv.conf'))
+
+        # drop machine-id file, if one exists
+        maybe_remove(os.path.join(instance_dir, 'etc', 'machine-id'))
+        # drop logfiles which we want to reset
+        maybe_remove(os.path.join(instance_dir, 'var', 'log', 'lastlog'))
+        maybe_remove(os.path.join(instance_dir, 'var', 'log', 'faillog'))
+
+    def _create_internal(self, mirror=None, components=None,
+                         extra_suites: list[str] = None, extra_source_lines: str = None,
+                         allow_recommends: bool = False,
+                         show_header: bool = True):
         ''' Create new container base image (internal method) '''
 
         if self.exists():
             print_error('An image already exists for this configuration. Can not create a new one.')
             return False
+        if not extra_suites:
+            extra_suites = []
 
         # ensure image location exists
         Path(self._gconf.osroots_dir).mkdir(parents=True, exist_ok=True)
@@ -250,7 +285,7 @@ class OSBase:
                 cmd.append(mirror)
 
             print_section('Bootstrap')
-            proc = subprocess.run(cmd)
+            proc = subprocess.run(cmd, check=False)
             if proc.returncode != 0:
                 return False
 
@@ -266,7 +301,9 @@ class OSBase:
                 if not mirror:
                     with open(sourceslist_fname, 'r') as f:
                         contents = f.read()
-                        matches = re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', contents)
+                        matches = re.findall(
+                            'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+',
+                            contents)
                         if not matches:
                             print_error('Unable to detect default APT repository URL (no regex matches).')
                             return False
@@ -276,10 +313,14 @@ class OSBase:
                         return False
 
                 if not components:
-                    components = ['main']  # FIXME: We should really be more clever here, e.g. depend on python-apt and parse sources.list properly
+                    # FIXME: We should really be more clever here, e.g. depend on python-apt
+                    # and parse sources.list properly
+                    components = ['main']
                 with open(sourceslist_fname, 'a') as f:
                     if self.has_base_suite:
-                        f.write('deb {mirror} {suite} {components}\n'.format(mirror=mirror, suite=self.suite, components=' '.join(components)))
+                        f.write('deb {mirror} {suite} {components}\n'.format(mirror=mirror,
+                                                                             suite=self.suite,
+                                                                             components=' '.join(components)))
 
                     if extra_suites:
                         f.write('\n')
@@ -287,22 +328,39 @@ class OSBase:
                             if esuite == self.suite or esuite == bootstrap_suite:
                                 # don't add existing suites multiple times
                                 continue
-                            f.write('deb {mirror} {esuite} {components}\n'.format(mirror=mirror, esuite=esuite, components=' '.join(components)))
+                            f.write('deb {mirror} {esuite} {components}\n'.format(mirror=mirror,
+                                                                                  esuite=esuite,
+                                                                                  components=' '.join(components)))
 
                     if extra_source_lines:
                         f.write('\n')
                         for line in extra_source_lines.split('\\n'):
                             f.write('{}\n'.format(line.strip()))
 
+            # write our default APT settings for this container
+            aptconf_fname = os.path.join(tdir, 'etc', 'apt', 'apt.conf.d', '99debspawn')
+            with open(aptconf_fname, 'w') as f:
+                # fail immediately with a proper exit code when e.g. apt update fails,
+                # so we can retry and don't silently use old packages
+                # (only available with newer APT versions)
+                f.write('APT::Update::Error-Mode "any";\n')
+                if not allow_recommends:
+                    f.write('APT::Install-Recommends "0";\n')
+                    f.write('APT::Install-Suggests "0";\n')
+
+            # delete unwanted files, especially resolv.conf as a broken one will
+            # mess with the next step
+            self._remove_unwanted_files(tdir)
+
             print_section('Configure')
-            if nspawn_run_helper_persist(self, tdir, self.new_nspawn_machine_name(), '--update') != 0:
+            if nspawn_run_helper_persist(self, tdir,
+                                         self.new_nspawn_machine_name(),
+                                         '--update',
+                                         build_uid=self._builder_uid) != 0:
                 return False
 
-            # drop machine-id file, if one exists
-            try:
-                os.remove(os.path.join(tdir, 'etc', 'machine-id'))
-            except OSError:
-                pass
+            # drop any unwanted files (again) before building the tarball
+            self._remove_unwanted_files(tdir)
 
             print_section('Creating Tarball')
             self._clear_image_tree(tdir)
@@ -314,7 +372,9 @@ class OSBase:
 
         return True
 
-    def create(self, mirror=None, components=None, extra_suites=[], extra_source_lines=None):
+    def create(self, mirror: str = None, components: list[str] = None, *,
+               extra_suites: list[str] = None, extra_source_lines: str = None,
+               allow_recommends: bool = False):
         ''' Create new container base image (internal method) '''
         ensure_root()
 
@@ -326,6 +386,7 @@ class OSBase:
                                     components=components,
                                     extra_suites=extra_suites,
                                     extra_source_lines=extra_source_lines,
+                                    allow_recommends=allow_recommends,
                                     show_header=True)
         if ret:
             print_info('Done.')
@@ -406,19 +467,19 @@ class OSBase:
 
         print_header('Updating container image')
 
-        with self.new_instance() as (instance_dir, machine_name):
+        with self.new_instance() as (instance_dir, _):
             # ensure helper script runner exists and is up to date
             self._copy_helper_script(instance_dir)
 
             print_section('Update')
-            if nspawn_run_helper_persist(self, instance_dir, self.new_nspawn_machine_name(), '--update') != 0:
+            if nspawn_run_helper_persist(self, instance_dir,
+                                         self.new_nspawn_machine_name(),
+                                         '--update',
+                                         build_uid=self._builder_uid) != 0:
                 return False
 
-            # drop machine-id file, if one exists
-            try:
-                os.remove(os.path.join(instance_dir, 'etc', 'machine-id'))
-            except OSError:
-                pass
+            # drop unwanted files from the image
+            self._remove_unwanted_files(instance_dir)
 
             print_section('Recreating tarball')
             self.make_instance_permanent(instance_dir)
@@ -457,6 +518,7 @@ class OSBase:
             components = cdata.get('Components')
             extra_suites = cdata.get('ExtraSuites', [])
             extra_source_lines = cdata.get('ExtraSourceLines')
+            allow_recommends = cdata.get('AllowRecommends', False)
 
         print_section('Deleting cache')
         cache_size = self._aptcache.clear()
@@ -479,6 +541,7 @@ class OSBase:
                                         components=components,
                                         extra_suites=extra_suites,
                                         extra_source_lines=extra_source_lines,
+                                        allow_recommends=allow_recommends,
                                         show_header=False)
         except Exception as e:
             print_error('Error while trying to create image: {}'.format(str(e)))
@@ -503,7 +566,7 @@ class OSBase:
             print_info('Recreation failed.')
             return False
 
-    def login(self, persistent=False, allowed=[]):
+    def login(self, persistent=False, allowed: list[str] = None):
         ''' Interactive shell login into the container '''
         ensure_root()
 
@@ -511,8 +574,9 @@ class OSBase:
             print_info('Can not enter "{}": The configuration does not exist.'.format(self.name))
             return False
 
-        print_header('Login (persistent changes) for {}'.format(self.name) if persistent else 'Login for {}'.format(self.name))
-        with self.new_instance() as (instance_dir, machine_name):
+        print_header('Login (persistent changes) for {}'.format(self.name)
+                     if persistent else 'Login for {}'.format(self.name))
+        with self.new_instance() as (instance_dir, _):
             # ensure helper script runner exists and is up to date
             self._copy_helper_script(instance_dir)
 
@@ -533,7 +597,7 @@ class OSBase:
         print_info('Done.')
         return True
 
-    def _copy_command_script_to_instance_dir(self, instance_dir: str, command_script: str) -> str:
+    def _copy_command_script_to_instance_dir(self, instance_dir: str, command_script: str) -> Optional[str]:
         '''
         Copy a script from the host to the current instance directory and make it
         executable.
@@ -554,7 +618,8 @@ class OSBase:
 
         return os.path.join('/srv', 'tmp', os.path.basename(host_script))
 
-    def run(self, command, build_dir, artifacts_dir, init_command=None, copy_command=False, header_msg=None, allowed=[]):
+    def run(self, command, build_dir, artifacts_dir, init_command=None, copy_command=False,
+            header_msg=None, allowed: list[str] = None):
         ''' Run an arbitrary command or script in the container '''
         ensure_root()
 
@@ -571,6 +636,7 @@ class OSBase:
                 import shlex
                 init_command = shlex.split(init_command)
         init_command = listify(init_command)
+        allowed = listify(allowed)
 
         # ensure we have absolute paths
         if build_dir:
@@ -586,19 +652,22 @@ class OSBase:
                 # ensure helper script runner exists and is up to date
                 self._copy_helper_script(instance_dir)
 
+                init_nspawn_flags = ['--bind={}:/srv/build/'.format(os.path.normpath(build_dir))]
                 if copy_command:
                     # copy initialization script from host to container
                     host_script = init_command[0]
                     init_command[0] = self._copy_command_script_to_instance_dir(instance_dir, host_script)
                     if not init_command[0]:
-                        print_error('Unable to find initialization script "{}", can not copy it to the container. Exiting.'.format(host_script))
+                        print_error(('Unable to find initialization script "{}", '
+                                     'can not copy it to the container. Exiting.').format(host_script))
                         return False
 
                 r = nspawn_run_helper_persist(self,
                                               instance_dir,
                                               machine_name,
                                               '--prepare-run',
-                                              '/srv')
+                                              '/srv',
+                                              build_uid=self._builder_uid)
                 if r != 0:
                     print_error('Container setup failed.')
                     return False
@@ -614,6 +683,7 @@ class OSBase:
                                        machine_name,
                                        '/srv',
                                        init_command,
+                                       init_nspawn_flags,
                                        allowed=filtered_allowed)
                 if r != 0:
                     return False
@@ -635,14 +705,16 @@ class OSBase:
                 host_script = command[0]
                 command[0] = self._copy_command_script_to_instance_dir(instance_dir, host_script)
                 if not command[0]:
-                    print_error('Unable to find script "{}", can not copy it to the container. Exiting.'.format(host_script))
+                    print_error(('Unable to find script "{}", can not copy it to the container. '
+                                 'Exiting.').format(host_script))
                     return False
 
             r = nspawn_run_helper_persist(self,
                                           instance_dir,
                                           machine_name,
                                           '--prepare-run',
-                                          '/srv')
+                                          '/srv',
+                                          build_uid=self._builder_uid)
             if r != 0:
                 print_error('Container setup failed.')
                 return False
