@@ -18,7 +18,7 @@
 from .log import print_info, print_warn, print_error, print_header, print_section
 from .env import colored_output_allowed, unicode_allowed
 from .command import safe_run, run_forwarded
-from .misc import temp_dir, cd, hardlink_or_copy, format_filesize
+from .misc import listify, temp_dir, cd, hardlink_or_copy, format_filesize
 
 __all__ = ['print_info',
            'print_warn',
@@ -29,6 +29,7 @@ __all__ = ['print_info',
            'unicode_allowed',
            'safe_run',
            'run_forwarded',
+           'listify',
            'temp_dir',
            'cd',
            'hardlink_or_copy',
