@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2016-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2016-2021 Matthias Klumpp <matthias@tenstral.net>
 # Copyright (C) 2012-2013 Paul Tagliamonte <paultag@debian.org>
 #
 # Licensed under the GNU Lesser General Public License Version 3
@@ -30,9 +30,11 @@ class SubprocessError(Exception):
         self.err = err
         self.ret = ret
         self.cmd = cmd
+        super(SubprocessError, self).__init__(
+            '%s: %d\n%s' % (str(self.cmd), self.ret, str(self.err)))
 
     def __str__(self):
-        return "%s: %d\n%s" % (str(self.cmd), self.ret, str(self.err))
+        return '%s: %d\n%s' % (str(self.cmd), self.ret, str(self.err))
 
 
 # Input may be a byte string, a unicode string, or a file-like object
@@ -92,4 +94,4 @@ def run_forwarded(command):
             sys.stdout.write(str(line, 'utf-8', 'replace'))
         return proc
     else:
-        return subprocess.run(command)
+        return subprocess.run(command, check=False)
