@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
+# Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
 #
 # Licensed under the GNU Lesser General Public License Version 3
 #
@@ -25,7 +25,8 @@ def ensure_tar_zstd():
     ''' Check if the required binaries for compression are available '''
 
     if not shutil.which('zstd'):
-        raise Exception('The "zsdt" binary was not found, we can not compress tarballs. Please install zstd to continue!')
+        raise Exception(('The "zsdt" binary was not found, we can not compress tarballs. '
+                         'Please install zstd to continue!'))
     if not shutil.which('tar'):
         raise Exception('The "tar" binary was not found, we can not create tarballs. Please install tar to continue!')
 
